@@ -825,7 +825,7 @@ PyArray_Clip(PyArrayObject *self, PyObject *min, PyObject *max, PyArrayObject *o
      */
     if (PyArray_ScalarKind(newdescr->type_num, NULL) >
         PyArray_ScalarKind(self->descr->type_num, NULL)) {
-        indescr = _array_small_type(newdescr, self->descr);
+        indescr = PyArray_PromoteTypes(newdescr, self->descr);
         func = indescr->f->fastclip;
         if (func == NULL) {
             return _slow_array_clip(self, min, max, out);
@@ -1082,4 +1082,3 @@ PyArray_Trace(PyArrayObject *self, int offset, int axis1, int axis2,
     Py_DECREF(diag);
     return ret;
 }
-
