@@ -10,6 +10,7 @@
 #include "scalartypes.c"
 #include "scalarapi.c"
 
+#include "datetime.c"
 #include "arraytypes.c"
 
 #include "hashdescr.c"
@@ -34,6 +35,12 @@
 #include "conversion_utils.c"
 #include "buffer.c"
 
+#include "nditer.c"
+#include "nditer_pywrap.c"
+#include "lowlevel_strided_loops.c"
+#include "dtype_transfer.c"
+#include "einsum.c"
+
 
 #ifndef Py_UNICODE_WIDE
 #include "ucsnarrow.c"
@@ -41,4 +48,6 @@
 
 #include "arrayobject.c"
 
+#include "numpymemoryview.c"
+
 #include "multiarraymodule.c"
