@@ -57,6 +57,7 @@ scalar_value(PyObject *scalar, PyArray_Descr *descr)
         CASE(ULONG, ULong);
         CASE(LONGLONG, LongLong);
         CASE(ULONGLONG, ULongLong);
+        CASE(HALF, Half);
         CASE(FLOAT, Float);
         CASE(DOUBLE, Double);
         CASE(LONGDOUBLE, LongDouble);
@@ -64,6 +65,8 @@ scalar_value(PyObject *scalar, PyArray_Descr *descr)
         CASE(CDOUBLE, CDouble);
         CASE(CLONGDOUBLE, CLongDouble);
         CASE(OBJECT, Object);
+        CASE(DATETIME, Datetime);
+        CASE(TIMEDELTA, Timedelta);
 #undef CASE
         case NPY_STRING:
             return (void *)PyString_AS_STRING(scalar);
@@ -91,6 +94,10 @@ scalar_value(PyObject *scalar, PyArray_Descr *descr)
                 _IFCASE(Int);
                 _IFCASE(Long);
                 _IFCASE(LongLong);
+                if _CHK(TimeInteger) {
+                    _IFCASE(Datetime);
+                    _IFCASE(Timedelta);
+                }
             }
             else {
                 /* Unsigned Integer */
@@ -104,6 +111,7 @@ scalar_value(PyObject *scalar, PyArray_Descr *descr)
         else {
             /* Inexact */
             if _CHK(Floating) {
+                _IFCASE(Half);
                 _IFCASE(Float);
                 _IFCASE(Double);
                 _IFCASE(LongDouble);
@@ -346,7 +354,19 @@ PyArray_ScalarFromObject(PyObject *object)
     if (PyArray_IsZeroDim(object)) {
         return PyArray_ToScalar(PyArray_DATA(object), object);
     }
-    if (PyInt_Check(object)) {
+    /*
+     * Booleans in Python are implemented as a subclass of integers,
+     * so PyBool_Check must be called before PyInt_Check.
+     */
+    if (PyBool_Check(object)) {
+        if (object == Py_True) {
+            PyArrayScalar_RETURN_TRUE;
+        }
+        else {
+            PyArrayScalar_RETURN_FALSE;
+        }
+    }
+    else if (PyInt_Check(object)) {
         ret = PyArrayScalar_New(Long);
         if (ret == NULL) {
             return NULL;
@@ -381,14 +401,6 @@ PyArray_ScalarFromObject(PyObject *object)
         }
         PyArrayScalar_VAL(ret, LongLong) = val;
     }
-    else if (PyBool_Check(object)) {
-        if (object == Py_True) {
-            PyArrayScalar_RETURN_TRUE;
-        }
-        else {
-            PyArrayScalar_RETURN_FALSE;
-        }
-    }
     return ret;
 }
 
@@ -510,6 +522,42 @@ PyArray_DescrFromScalar(PyObject *sc)
         return descr;
     }
 
+    if (PyArray_IsScalar(sc, TimeInteger)) {
+        PyObject *cobj;
+        PyArray_DatetimeMetaData *dt_data;
+
+        dt_data = _pya_malloc(sizeof(PyArray_DatetimeMetaData));
+        if (PyArray_IsScalar(sc, Datetime)) {
+            descr = PyArray_DescrNewFromType(PyArray_DATETIME);
+            memcpy(dt_data, &((PyDatetimeScalarObject *)sc)->obmeta,
+                   sizeof(PyArray_DatetimeMetaData));
+        }
+        else {
+            /* Timedelta */
+            descr = PyArray_DescrNewFromType(PyArray_TIMEDELTA);
+            memcpy(dt_data, &((PyTimedeltaScalarObject *)sc)->obmeta,
+                   sizeof(PyArray_DatetimeMetaData));
+        }
+        cobj = NpyCapsule_FromVoidPtr((void *)dt_data, simple_capsule_dtor);
+
+        /* Add correct meta-data to the data-type */
+        if (descr == NULL) {
+            Py_DECREF(cobj);
+            return NULL;
+        }
+        Py_XDECREF(descr->metadata);
+        if ((descr->metadata = PyDict_New()) == NULL) {
+            Py_DECREF(descr);
+            Py_DECREF(cobj);
+            return NULL;
+        }
+
+        /* Assume this sets a new reference to cobj */
+        PyDict_SetItemString(descr->metadata, NPY_METADATA_DTSTR, cobj);
+        Py_DECREF(cobj);
+        return descr;
+    }
+
     descr = PyArray_DescrFromTypeObject((PyObject *)Py_TYPE(sc));
     if (descr->elsize == 0) {
         PyArray_DESCR_REPLACE(descr);
@@ -614,6 +662,22 @@ PyArray_Scalar(void *data, PyArray_Descr *descr, PyObject *base)
     if (obj == NULL) {
         return NULL;
     }
+    if (PyTypeNum_ISDATETIME(type_num)) {
+        /*
+         * We need to copy the resolution information over to the scalar
+         * Get the void * from the metadata dictionary
+         */
+        PyObject *cobj;
+        PyArray_DatetimeMetaData *dt_data;
+        cobj = PyDict_GetItemString(descr->metadata, NPY_METADATA_DTSTR);
+
+/* FIXME
+ * There is no error handling here.
+ */
+        dt_data = NpyCapsule_AsVoidPtr(cobj);
+        memcpy(&(((PyDatetimeScalarObject *)obj)->obmeta), dt_data,
+               sizeof(PyArray_DatetimeMetaData));
+    }
     if (PyTypeNum_ISFLEXIBLE(type_num)) {
         if (type_num == PyArray_STRING) {
             destptr = PyString_AS_STRING(obj);
