@@ -1,14 +1,13 @@
 import re
 import os
 import sys
-import new
+import types
 from copy import copy
 
 from distutils.ccompiler import *
 from distutils import ccompiler
 from distutils.errors import DistutilsExecError, DistutilsModuleError, \
                              DistutilsPlatformError
-
 from distutils.sysconfig import customize_compiler
 from distutils.version import LooseVersion
 
@@ -16,17 +15,15 @@ from numpy.distutils import log
 from numpy.distutils.exec_command import exec_command
 from numpy.distutils.misc_util import cyg2win32, is_sequence, mingw32, \
                                       quote_args, msvc_on_amd64
+from numpy.distutils.compat import get_exception
 
-# hack to set compiler optimizing options. Needs to integrated with something.
-import distutils.sysconfig
-_old_init_posix = distutils.sysconfig._init_posix
-def _new_init_posix():
-    _old_init_posix()
-    distutils.sysconfig._config_vars['OPT'] = '-Wall -g -O0'
-#distutils.sysconfig._init_posix = _new_init_posix
 
 def replace_method(klass, method_name, func):
-    m = new.instancemethod(func, None, klass)
+    if sys.version_info[0] < 3:
+        m = types.MethodType(func, None, klass)
+    else:
+        # Py3k does not have unbound method anymore, MethodType does not work
+        m = lambda self, *args, **kw: func(self, *args, **kw)
     setattr(klass, method_name, m)
 
 # Using customized CCompiler.spawn.
@@ -61,13 +58,12 @@ def CCompiler_spawn(self, cmd, display=None):
     if s:
         if is_sequence(cmd):
             cmd = ' '.join(list(cmd))
-        print o
+        print(o)
         if re.search('Too many open files', o):
             msg = '\nTry rerunning setup command until build succeeds.'
         else:
             msg = ''
-        raise DistutilsExecError,\
-              'Command "%s" failed with exit status %d%s' % (cmd, s, msg)
+        raise DistutilsExecError('Command "%s" failed with exit status %d%s' % (cmd, s, msg))
 
 replace_method(CCompiler, 'spawn', CCompiler_spawn)
 
@@ -109,8 +105,7 @@ def CCompiler_object_filenames(self, source_filenames, strip_dir=0, output_dir='
             d = os.path.basename(os.path.abspath(d))
             base = d + base[i:]
         if ext not in self.src_extensions:
-            raise UnknownFileError, \
-                  "unknown file type '%s' (from '%s')" % (ext, src_name)
+            raise UnknownFileError("unknown file type '%s' (from '%s')" % (ext, src_name))
         if strip_dir:
             base = os.path.basename(base)
         obj_name = os.path.join(output_dir,base + self.obj_extension)
@@ -162,7 +157,11 @@ def CCompiler_compile(self, sources, output_dir=None, macros=None,
     # method to support pre Python 2.3 distutils.
     if not sources:
         return []
-    from fcompiler import FCompiler
+    # FIXME:RELATIVE_IMPORT
+    if sys.version_info[0] < 3:
+        from fcompiler import FCompiler
+    else:
+        from numpy.distutils.fcompiler import FCompiler
     if isinstance(self, FCompiler):
         display = []
         for fc in ['f77','f90','fix']:
@@ -297,10 +296,10 @@ def CCompiler_show_customization(self):
     except:
         pass
     if log._global_log.threshold<2:
-        print '*'*80
-        print self.__class__
-        print _compiler_to_string(self)
-        print '*'*80
+        print('*'*80)
+        print(self.__class__)
+        print(_compiler_to_string(self))
+        print('*'*80)
 
 replace_method(CCompiler, 'show_customization', CCompiler_show_customization)
 
@@ -340,12 +339,7 @@ def CCompiler_customize(self, dist, need_cxx=0):
     if need_cxx:
         # In general, distutils uses -Wstrict-prototypes, but this option is
         # not valid for C++ code, only for C.  Remove it if it's there to
-        # avoid a spurious warning on every compilation.  All the default
-        # options used by distutils can be extracted with:
-
-        # from distutils import sysconfig
-        # sysconfig.get_config_vars('CC', 'CXX', 'OPT', 'BASECFLAGS',
-        # 'CCSHARED', 'LDSHARED', 'SO')
+        # avoid a spurious warning on every compilation.
         try:
             self.compiler_so.remove('-Wstrict-prototypes')
         except (AttributeError, ValueError):
@@ -394,6 +388,9 @@ def simple_version_match(pat=r'[-.\d]+', ignore='', start=''):
 
     """
     def matcher(self, version_string):
+        # version string may appear in the second line, so getting rid
+        # of new lines:
+        version_string = version_string.replace('\n',' ')
         pos = 0
         if start:
             m = re.match(start, version_string)
@@ -497,7 +494,14 @@ compiler_class['intel'] = ('intelccompiler','IntelCCompiler',
                            "Intel C Compiler for 32-bit applications")
 compiler_class['intele'] = ('intelccompiler','IntelItaniumCCompiler',
                            "Intel C Itanium Compiler for Itanium-based applications")
-ccompiler._default_compilers += (('linux.*','intel'),('linux.*','intele'))
+compiler_class['intelem'] = ('intelccompiler','IntelEM64TCCompiler',
+                             "Intel C Compiler for 64-bit applications")
+compiler_class['pathcc'] = ('pathccompiler','PathScaleCCompiler',
+                            "PathScale Compiler for SiCortex-based applications")
+ccompiler._default_compilers += (('linux.*','intel'),
+                                 ('linux.*','intele'),
+                                 ('linux.*','intelem'),
+                                 ('linux.*','pathcc'))
 
 if sys.platform == 'win32':
     compiler_class['mingw32'] = ('mingw32ccompiler', 'Mingw32CCompiler',
@@ -528,27 +532,27 @@ def new_compiler (plat=None,
         msg = "don't know how to compile C/C++ code on platform '%s'" % plat
         if compiler is not None:
             msg = msg + " with '%s' compiler" % compiler
-        raise DistutilsPlatformError, msg
+        raise DistutilsPlatformError(msg)
     module_name = "numpy.distutils." + module_name
     try:
         __import__ (module_name)
-    except ImportError, msg:
+    except ImportError:
+        msg = str(get_exception())
         log.info('%s in numpy.distutils; trying from distutils',
                  str(msg))
         module_name = module_name[6:]
         try:
             __import__(module_name)
-        except ImportError, msg:
-            raise DistutilsModuleError, \
-                  "can't compile C/C++ code: unable to load module '%s'" % \
-                  module_name
+        except ImportError:
+            msg = str(get_exception())
+            raise DistutilsModuleError("can't compile C/C++ code: unable to load module '%s'" % \
+                  module_name)
     try:
         module = sys.modules[module_name]
         klass = vars(module)[class_name]
     except KeyError:
-        raise DistutilsModuleError, \
-              ("can't compile C/C++ code: unable to find class '%s' " +
-               "in module '%s'") % (class_name, module_name)
+        raise DistutilsModuleError(("can't compile C/C++ code: unable to find class '%s' " +
+               "in module '%s'") % (class_name, module_name))
     compiler = klass(None, dry_run, force)
     log.debug('new_compiler returns %s' % (klass))
     return compiler
@@ -596,7 +600,7 @@ _squote_re = re.compile(r"'(?:[^'\\]|\\.)*'")
 _dquote_re = re.compile(r'"(?:[^"\\]|\\.)*"')
 _has_white_re = re.compile(r'\s')
 def split_quoted(s):
-    s = string.strip(s)
+    s = s.strip()
     words = []
     pos = 0
 
@@ -609,7 +613,7 @@ def split_quoted(s):
 
         if s[end] in string.whitespace: # unescaped, unquoted whitespace: now
             words.append(s[:end])       # we definitely have a word delimiter
-            s = string.lstrip(s[end:])
+            s = s[end:].lstrip()
             pos = 0
 
         elif s[end] == '\\':            # preserve whatever is being escaped;
@@ -623,12 +627,10 @@ def split_quoted(s):
             elif s[end] == '"':         # slurp doubly-quoted string
                 m = _dquote_re.match(s, end)
             else:
-                raise RuntimeError, \
-                      "this can't happen (bad char '%c')" % s[end]
+                raise RuntimeError("this can't happen (bad char '%c')" % s[end])
 
             if m is None:
-                raise ValueError, \
-                      "bad string (mismatched %s quotes?)" % s[end]
+                raise ValueError("bad string (mismatched %s quotes?)" % s[end])
 
             (beg, end) = m.span()
             if _has_white_re.search(s[beg+1:end-1]):
