@@ -11,7 +11,6 @@ takes keyword arguments for (re-)defining environment variables.
 Provides functions:
   exec_command  --- execute command in a specified directory and
                     in the modified environment.
-  splitcmdline  --- inverse of ' '.join(argv)
   find_executable --- locate a command using info from environment
                     variable PATH. Equivalent to posix `which`
                     command.
@@ -50,17 +49,19 @@ __all__ = ['exec_command','find_executable']
 
 import os
 import sys
+import shlex
 
 from numpy.distutils.misc_util import is_sequence, make_temp_file
 from numpy.distutils import log
+from numpy.distutils.compat import get_exception
+
+from numpy.compat import open_latin1
 
 def temp_file_name():
     fo, name = make_temp_file()
     fo.close()
     return name
 
-############################################################
-
 def get_pythonexe():
     pythonexe = sys.executable
     if os.name in ['nt','dos']:
@@ -70,57 +71,11 @@ def get_pythonexe():
         assert os.path.isfile(pythonexe), '%r is not a file' % (pythonexe,)
     return pythonexe
 
-############################################################
-
 def splitcmdline(line):
-    """ Inverse of ' '.join(sys.argv).
-    """
-    log.debug('splitcmdline(%r)' % (line))
-    lst = []
-    flag = 0
-    s,pc,cc = '','',''
-    for nc in line+' ':
-        if flag==0:
-            flag = (pc != '\\' and \
-                     ((cc=='"' and 1) or (cc=="'" and 2) or \
-                       (cc==' ' and pc!=' ' and -2))) or flag
-        elif flag==1:
-            flag = (cc=='"' and pc!='\\' and nc==' ' and -1) or flag
-        elif flag==2:
-            flag = (cc=="'" and pc!='\\' and nc==' ' and -1) or flag
-        if flag!=-2:
-            s += cc
-        if flag<0:
-            flag = 0
-            s = s.strip()
-            if s:
-                lst.append(s)
-                s = ''
-        pc,cc = cc,nc
-    else:
-        s = s.strip()
-        if s:
-            lst.append(s)
-    log.debug('splitcmdline -> %r' % (lst))
-    return lst
-
-def test_splitcmdline():
-    l = splitcmdline('a   b  cc')
-    assert l==['a','b','cc'], repr(l)
-    l = splitcmdline('a')
-    assert l==['a'], repr(l)
-    l = splitcmdline('a "  b  cc"')
-    assert l==['a','"  b  cc"'], repr(l)
-    l = splitcmdline('"a bcc"  -h')
-    assert l==['"a bcc"','-h'], repr(l)
-    l = splitcmdline(r'"\"a \" bcc" -h')
-    assert l==[r'"\"a \" bcc"','-h'], repr(l)
-    l = splitcmdline(" 'a bcc'  -h")
-    assert l==["'a bcc'",'-h'], repr(l)
-    l = splitcmdline(r"'\'a \' bcc' -h")
-    assert l==[r"'\'a \' bcc'",'-h'], repr(l)
-
-############################################################
+    import warnings
+    warnings.warn('splitcmdline is deprecated; use shlex.split',
+                  DeprecationWarning)
+    return shlex.split(line)
 
 def find_executable(exe, path=None, _cache={}):
     """Return full path of a executable or None.
@@ -295,13 +250,13 @@ def _exec_command_posix( command,
             return _exec_command(command, use_shell=use_shell, **env)
 
     if stsfile is not None:
-        f = open(stsfile,'r')
+        f = open_latin1(stsfile,'r')
         status_text = f.read()
         status = int(status_text)
         f.close()
         os.remove(stsfile)
 
-    f = open(tmpfile,'r')
+    f = open_latin1(tmpfile,'r')
     text = f.read()
     f.close()
     os.remove(tmpfile)
@@ -340,12 +295,12 @@ def _exec_command_python(command,
         raise RuntimeError("%r failed" % (cmd,))
     os.remove(cmdfile)
 
-    f = open(stsfile,'r')
+    f = open_latin1(stsfile,'r')
     status = int(f.read())
     f.close()
     os.remove(stsfile)
 
-    f = open(outfile,'r')
+    f = open_latin1(outfile,'r')
     text = f.read()
     f.close()
     os.remove(outfile)
@@ -379,7 +334,7 @@ def _exec_command( command, use_shell=None, use_tee = None, **env ):
         if is_sequence(command):
             argv = command[:]
         else:
-            argv = splitcmdline(command)
+            argv = shlex.split(command)
 
     if hasattr(os,'spawnvpe'):
         spawn_command = os.spawnvpe
@@ -426,7 +381,8 @@ def _exec_command( command, use_shell=None, use_tee = None, **env ):
         os.dup2(fout.fileno(),se_fileno)
     try:
         status = spawn_command(os.P_WAIT,argv0,argv,os.environ)
-    except OSError,errmess:
+    except OSError:
+        errmess = str(get_exception())
         status = 999
         sys.stderr.write('%s: %s'%(errmess,argv[0]))
 
@@ -436,14 +392,14 @@ def _exec_command( command, use_shell=None, use_tee = None, **env ):
     os.dup2(se_dup,se_fileno)
 
     fout.close()
-    fout = open(outfile,'r')
+    fout = open_latin1(outfile,'r')
     text = fout.read()
     fout.close()
     os.remove(outfile)
 
     if using_command:
         ferr.close()
-        ferr = open(errfile,'r')
+        ferr = open_latin1(errfile,'r')
         errmess = ferr.read()
         ferr.close()
         os.remove(errfile)
@@ -456,14 +412,14 @@ def _exec_command( command, use_shell=None, use_tee = None, **env ):
                 text = text + '\n'
             #text = '%sCOMMAND %r FAILED: %s' %(text,command,errmess)
             text = text + errmess
-            print errmess
+            print (errmess)
     if text[-1:]=='\n':
         text = text[:-1]
     if status is None:
         status = 0
 
     if use_tee:
-        print text
+        print (text)
 
     return status, text
 
@@ -542,7 +498,7 @@ def test_nt(**kws):
     s,o=exec_command('%s -c "print \'Heipa\'"' % pythonexe)
     assert s==0 and o=='Heipa',(s,o)
 
-    print 'ok'
+    print ('ok')
 
 def test_posix(**kws):
     s,o=exec_command("echo Hello",**kws)
@@ -590,7 +546,7 @@ def test_posix(**kws):
     s,o=exec_command('python -c "print \'Heipa\'"',**kws)
     assert s==0 and o=='Heipa',(s,o)
 
-    print 'ok'
+    print ('ok')
 
 def test_execute_in(**kws):
     pythonexe = get_pythonexe()
@@ -608,31 +564,30 @@ def test_execute_in(**kws):
                        execute_in = tmpdir,**kws)
     assert s==0 and o=='Hello',(s,o)
     os.remove(tmpfile)
-    print 'ok'
+    print ('ok')
 
 def test_svn(**kws):
     s,o = exec_command(['svn','status'],**kws)
     assert s,(s,o)
-    print 'svn ok'
+    print ('svn ok')
 
 def test_cl(**kws):
     if os.name=='nt':
         s,o = exec_command(['cl','/V'],**kws)
         assert s,(s,o)
-        print 'cl ok'
+        print ('cl ok')
 
 if os.name=='posix':
     test = test_posix
 elif os.name in ['nt','dos']:
     test = test_nt
 else:
-    raise NotImplementedError,'exec_command tests for '+os.name
+    raise NotImplementedError('exec_command tests for ', os.name)
 
 ############################################################
 
 if __name__ == "__main__":
 
-    test_splitcmdline()
     test(use_tee=0)
     test(use_tee=1)
     test_execute_in(use_tee=0)
